;-- Keyboard Emulation

;===============================================================
.DATA

 EXTRN keybd:word	  ;-- Keyboard
 EXTRN shift:word
 EXTRN altgr:word
 EXTRN xkeys:word
 EXTRN xshft:word

 EXTRN KeyCount:word
 EXTRN KeyStart:word
 EXTRN KeyState:byte  ;-- Extanded(1),XShift(2),AltGr(4),LShift(8),RShift(16)

 EXTRN KeyBuffer:byte ;-- db 256 dup(255)

 EXTRN KeyDown:byte   ;-- nb of key down ...not used...(look at KEYBOARD.PAS)

 OldInt09  dd 0

 KeyCode db 128 dup (?)
 LastKey db 0

;===============================================================
.CODE

BEGINP UpdateKeyboard

 MOV DS:[BX+Offset KeyCode],AH ; Save C64Key

; no C=64 key
 CMP AH,0FFh
 JE  KeyDone

; Joystick Emulation
 TEST AH,40h
 JZ NotJoystick
  AND AH,255-40h
  OR  JoyState,AH
  TEST AL,80h
  JNZ KeyDone
  NOT AH
  AND JoyState,AH
 RET
 NotJoyStick:

; C=64 Shift

 TEST AH,80h
 JZ SetupColRow

  AND AH,7Fh
  TEST AL,80h
  JNZ ReleaseShift
   AND [KeyMatrix+1],01111111b ; not (1 shl 7)
   AND [RevMatrix+7],11111101b ; not (1 shl 1)
   JMP SetupColRow
  ReleaseShift:
   OR  [KeyMatrix+1],10000000b ; (1 shl 7)
   OR  [RevMatrix+7],00000010b ; (1 shl 1)

  SetupColRow:
   MOV CH,AH
   MOV CL,CH
   SHR CH,3 ; col
   AND CL,7 ; row

   MOV BX,0101h
   SHL BL,CL
  XCHG CL,CH
   SHL BH,CL

   TEST AL,80h
   MOV AX,BX

   JNZ ReleaseKey
    XOR BH,BH
    MOV BL,CL
    NOT AX
    AND [BX+Offset KeyMatrix],AL
    MOV BL,CH
    AND [BX+Offset RevMatrix],AH
   JMP KeyDone
   ReleaseKey:
    XOR BH,BH
    MOV BL,CL
    OR  [BX+Offset KeyMatrix],AL
    MOV BL,CH
    OR  [BX+Offset RevMatrix],AH
   RET
UpdateKeyboard ENDP

BEGINP CheckKeyboard
 CLI
 MOV BX,KeyStart
 CMP BX,KeyCount
 JNE ProcessKey
 STI
 RET
CheckKeyboard ENDP

BEGINP ProcessKey
 INC BL
 MOV AL,[BX+Offset KeyBuffer]
 MOV KeyStart,BX
 STI

 MOV BX,AX
 AND BX,7Fh

 TEST KeyState,1 ; Extended
 JZ StandardKey

  AND KeyState,255-1

  CMP BL,2Ah
  JE  SetXShift
  CMP BL,38h
  JE  SetAltGr

  CMP AL,49h ; PageUp
  JE  SetJoy1

  CMP AL,51h ; PageDown
  JE  SetJoy2

  TEST AL,80h
  JZ xKey_Down
   MOV AH,DS:[BX+Offset KeyCode]
   DEC KeyDown
   JNS UpdateKeyboard
   MOV KeyDown,0
   JMP UpdateKeyboard
  xKey_Down:
  INC KeyDown

  TEST KeyState,2 ; XShift
  JZ SetXKey
   MOV AH,[BX+Offset xShft-1]
   JMP UpdateKeyboard
  SetXKey:
   MOV AH,[BX+Offset xKeys-1]
  JMP UpdateKeyboard

  SetXShift:
   AND KeyState,255-2
   TEST AL,80h
   JNZ KeyDone
   OR KeyState,2
  RET

  SetAltGr:
   AND KeyState,255-4
   TEST AL,80h
   JNZ KeyDone
   OR KeyState,4
  RET

  SetJoy1:
   MOV Joystick,1
;   IFE DEBUG
;    CALLP Joy1Active
;   ENDIF
  RET

  SetJoy2:
   MOV Joystick,2
;   IFE DEBUG
;    CALLP Joy2Active
;   ENDIF
  RET

 StandardKey:

  CMP BL,2Ah
  JE  SetLShift
  CMP BL,36h
  JE  SetRShift
  CMP AL,0E0h  ; !!
  JE  SetXState
  CMP AL,0E1h
  JE  DoPause

  CMP AL,01
  JNE Not_Restore
   OR CIA2,NMIFlag ; CIA2.ASM
  Not_Restore:

  CMP AL,43h
  JE KeyF9

  CMP AL,44h
  JE KeyF10

  CMP AL,57h
  JE KeyF11

;  CMP AL,58h
;  JE KeyF12

 NotEscKey:


 ; To avoid invalide "keyup" (Press Shift+A, Release Shift, Release A)
  TEST AL,80h
  JZ Key_Down
   MOV AH,DS:[BX+Offset KeyCode]
   DEC KeyDown
   JNS UpdateKeyboard
   MOV KeyDown,0
   JMP UpdateKeyboard
  Key_Down:
  INC KeyDown

  TEST KeyState,8+16
  JZ NoShift
   MOV AH,[BX+Offset Shift-1]
   JMP UpdateKeyboard
  NoShift:
  TEST KeyState,4
  JZ NoAltGr
   MOV AH,[BX+Offset AltGr-1]
   JMP UpdateKeyboard
  NoAltGr:
   MOV AH,[BX+Offset Keybd-1]
  JMP UpdateKeyboard

  SetLShift:
   AND KeyState,255-8
   TEST AL,80h
   JNZ KeyDone
   OR KeyState,8
  RET

  SetRShift:
   AND KeyState,255-16
   TEST AL,80h
   JNZ KeyDone
   OR KeyState,16
  RET

  KeyF9:
   CALLP ChangeDisk
  RET

  KeyF10:
  IFE DEBUG
   CALLP StatusInfo
  ENDIF
  RET

  KeyF11:
   Call Reset
;   Call NewBank - done by Kernal
;   Call NewVideoMode - done by Kernal
   POP AX ; RET from CheckKeyboard
   POP AX ; RET from UpdateRaster - added in v2.5 !
  JMP ConfigMem ; Restore ROMS !!!

;  KeyF12:
;   MOV word ptr CS:ExitPatch,01EBh ; JMP +1
;  RET

  DoPause:
   CALLP Pause
  RET

  SetXState:
   OR KeyState,1

 KeyDone:

 RET
ProcessKey ENDP

Align 2
KeyboardHandler:
 PUSH AX
 PUSH BX
 PUSH DX
 PUSH DS

 MOV AX,SEG @DATA
 MOV DS,AX

 MOV  DX,60h ; Scan:=Port[$60];
 IN   AL,DX
 MOV  BL,AL

 ; BIOS Keys
 AND  AL,7Fh
 CMP  AL,45h ; NumLock
 JE   BIOSKey
 CMP  BL,0FAh ; ??? leds
 JE   BIOSKey

 INC  DX
 IN   AL,DX  ; dummy:=Port[$61];
 MOV  AH,AL
 OR   AL,80h ; Port[$61]:=dummy or $80;
 OUT  DX,AL
 MOV  AL,AH  ; Port[$61]:=dummy;
 OUT  DX,AL

 MOV  AL,BL

 CMP AL,58h ; F12
 JNE Not_F12
   MOV word ptr CS:ExitPatch,01EBh ; JMP +1
 Not_F12:

 MOV  BX,KeyCount
 INC  BL
 CMP  BX,KeyStart
 JE ToManyKeys
  MOV [BX+Offset KeyBuffer],AL
  MOV KeyCount,BX
 ToManyKeys:

 CMP LastKey,0E0h
 JNE Not_CtrlBreak
 CMP AL,46h ; Ctrl+Break
 JNE Not_CtrlBreak

  MOV word ptr CS:ExitPatch,01EBh ; JMP +1
  Call RemoveKeyboard
;  Call TextMode
;  MOV AX,4C00h
;  INT 21h

 Not_CtrlBreak:
 MOV LastKey,AL


 MOV DX,20h
 MOV AL,DL
 OUT DX,AL

 ExitKeyHandler:

 POP DS
 POP DX
 POP BX
 POP AX
IRET

BEGINP BIOSKey
 PUSHF
 CALL OldInt09
 JMP ExitKeyHandler
ENDP

BEGINP InstallKeyboard

  mov ax,3509h ; Keyboard INT
  int 21h
  mov word ptr OldInt09,bx
  mov word ptr (OldInt09+2),es

  push ds
   mov ax,cs
   mov ds,ax
   mov dx,offset KeyboardHandler
   mov ax,2509h
   int 21h
  pop  ds

  RET

InstallKeyboard ENDP

BEGINP RemoveKeyboard
 push ds
  lds dx,OldInt09
  mov ax,2509h
  int 21h
 pop ds
 mov ax,0C00h ; flush DOS Keyboard Buffer
 int 21h
 RET
RemoveKeyboard ENDP

